from typing import List
from enum import Enum
from abc import ABC
from deepteam.metrics import BaseRedTeamingMetric


class BaseVulnerability(ABC):
    metric: BaseRedTeamingMetric

    def __init__(self, types: List[Enum]):
        """
        Initialize a Vulnerability with a list of specific types.
        :param types: A list of Enum types representing vulnerabilities.
        """
        self.types = types

    def get_types(self) -> List[Enum]:
        """
        Get the list of Enum types.
        :return: List of Enum types.
        """
        return self.types

    def get_values(self) -> List[str]:
        """
        Get the list of values for the Enum types.
        :return: List of string values from the Enum types.
        """
        return [t.value for t in self.types]

    def assess(self):
        pass

    async def a_assess(self):
        pass

    def simulate_attacks(self, purpose: str = None):
        pass

    async def a_simulate_attacks(self, purpose: str = None):
        pass

    def _get_metric(self) -> BaseRedTeamingMetric:
        """
        Get the corresponding metric of the vulnerability.
        :return: List of Enum types.
        """
        pass

    def get_name(self) -> str:
        return self.__class__.__name__

    def __repr__(self):
        """
        Represent the class by listing the Enum types.
        :return: String representation of the Vulnerability class.
        """
        return f"{self.__class__.__name__} (types={self.types})"
